import { ChangeDetectorRef, Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { AppraisalFormModel, MyAppraisalFormModel } from 'src/app/shared/model/appraisal-form.model';
import { AppraisalModel, MyAppraisalModel } from 'src/app/shared/model/appraisal.model';
import { EmployeeModel, MyEmployeeModel } from 'src/app/shared/model/employee.model';
import { IdpDevelopmentModel, MyIdpDevelopmentModel } from 'src/app/shared/model/idp-development.model';
import { MySettingAssessmentModel, SettingAssessmentModel } from 'src/app/shared/model/setting-assessment.model';
import { AppraisalService } from 'src/app/shared/services/appraisal.service';
import { EmployeeService } from 'src/app/shared/services/employee.service';
import { IdpDevelopmentService } from 'src/app/shared/services/idp-development.service';
import { SettingAssessmentService } from 'src/app/shared/services/setting-assessment.service';
import Swal from 'sweetalert2';

export interface BiModel {
  name: string,
  tools: string[],
  degree: string
}

@Component({
  selector: 'app-self-evaluation',
  templateUrl: './self-evaluation.component.html',
  styleUrls: ['./self-evaluation.component.scss']
})
export class SelfEvaluationComponent implements OnInit {
  pathTitle = ['การประเมินผล', 'ประเมินตนเอง']
  hoveredCode: string | null = null;
  checkSheet: {
    score: string[],
    condition: string[]
  }[] = [{
    score: ["หากได้คะแนน 90 - 100% ถือว่า Gap +1", "หากได้คะแนน 80 - 89% ถือว่า ไม่มี Gap", "หากได้คะแนน 60 - 79% ถือว่า Gap - 1", "หากได้คะแนน 40 - 59% ถือว่า Gap -2", "หากได้คะแนน 0 - 39% ถือว่า Gap - 3"],
    condition: ["1.หากได้คะแนนสูงกว่า 80% แต่มี 3 2 หรือ 1 ด้วย ถือว่า Gap -1", "2.หากได้คะแนนต่ำกว่า 80% แต่มี 4 และ 5 ให้คิด Gap ตาม %", "3.คะแนนต่ำกว่า 80% แต่มี 2 และ 1 ให้คิด Gap ตาม %"],
  }]
  @ViewChild('selfEvaluation1_2', { static: true }) selfEvaluation1_2!: TemplateRef<any>;
  @ViewChild('selfEvaluation1_2Edit', { static: true }) selfEvaluation1_2Edit!: TemplateRef<any>;
  currentTemplate: TemplateRef<any> = this.selfEvaluation1_2

  appraisalCompentency: { loading: boolean, select: AppraisalModel, dataList: { check: boolean, data: AppraisalModel }[] } = { loading: false, select: new MyAppraisalModel(), dataList: [] }
  appraisalCompentencyForm: { loading: boolean, data: AppraisalFormModel } = { loading: false, data: new MyAppraisalFormModel() }
  appraisalPms: { loading: boolean, select: AppraisalModel, dataList: { check: boolean, data: AppraisalModel }[] } = { loading: false, select: new MyAppraisalModel(), dataList: [] }
  employee: { loading: boolean, data: EmployeeModel } = { loading: false, data: new MyEmployeeModel() }
  setting: { loading: boolean, data: SettingAssessmentModel } = { loading: false, data: new MySettingAssessmentModel() }
  currentDate = new Date()
  constructor(private appraisalService: AppraisalService,
    private employeeService: EmployeeService,
    private cdr: ChangeDetectorRef,
    private settingAssessmentService: SettingAssessmentService
  ) {
  }
  ngOnInit(): void {
    this.currentTemplate = this.selfEvaluation1_2
    this.getProfile()
    this.getAppraisalCompentencyList()
    this.getSettingList()
  }


  getSettingList() {
    this.setting.loading = true
    this.settingAssessmentService.get().subscribe({
      next: response => {
        this.setting.data = new MySettingAssessmentModel(response)
        this.setting.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.setting.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  getProfile() {
    this.employee.loading = true
    this.employeeService.getProfile().subscribe({
      next: response => {
        this.employee.data = new MyEmployeeModel(response)
        this.employee.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.employee.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  getAppraisalCompentencyList() {
    this.appraisalCompentency.loading = true
    this.appraisalService.getCompetencyList().subscribe({
      next: response => {
        this.appraisalCompentency.dataList = response.map(x => ({ check: false, data: new MyAppraisalModel(x) }))
        this.appraisalCompentency.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.appraisalCompentency.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  appraisalCompentencyFilter() {
    return this.appraisalCompentency.dataList
  }
  selectAppraisalCompentency(data: AppraisalModel) {
    this.appraisalCompentency.select = new MyAppraisalModel(data)
  }


  getAppraisalCompentencyForm() {
    this.appraisalCompentencyForm.loading = true
    this.appraisalService.getFormCompentencyById(this.appraisalCompentency.select.masfromEvaluationRoundModel.evaluationRoundId, this.employee.data.employeeId).subscribe({
      next: response => {
        this.appraisalCompentencyForm.data = new MyAppraisalFormModel(response)
        this.appraisalCompentencyForm.data = response
        this.calnumberCheck()
        this.appraisalCompentencyForm.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.appraisalCompentencyForm.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  appraisalCompentencyFormFilter() {
    return this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList
  }


  appraisalPmsFilter() {
    return this.appraisalPms.dataList
  }
  showTools(data: string[]) {
    return data.join("/")
  }

  saveDraft() {
    Swal.fire({
      iconHtml: `
          <div class="flex items-center justify-center rounded-full !h-80px !w-80px" style="background-color: #E8F8EE;">
              <svg width="39" height="39" viewBox="0 0 39 39" fill="none" xmlns="http://www.w3.org/2000/svg">
                  <rect width="39" height="39" fill="#D2D2D2"/>
                  <g id="Component">
                      <g id="alert cart">
                          <g id="mdi:file-export">
                              <circle cx="22.5" cy="19.5" r="33.5" fill="#E8F8EE"/>
                              <path d="M9.75 3.25C8.88805 3.25 8.0614 3.59241 7.4519 4.2019C6.84241 4.8114 6.5 5.63805 6.5 6.5V32.5C6.5 33.362 6.84241 34.1886 7.4519 34.7981C8.0614 35.4076 8.88805 35.75 9.75 35.75H29.25C30.112 35.75 30.9386 35.4076 31.5481 34.7981C32.1576 34.1886 32.5 33.362 32.5 32.5V13L22.75 3.25M21.125 5.6875L30.0625 14.625H21.125M14.5113 19.8575H26V31.3463L22.555 27.9013L17.9563 32.5L13.3575 27.9013L17.9563 23.3188" 
                              fill="#1DBE5A"/>
                          </g>
                      </g>
                  </g>
              </svg>
          </div>
          `,
      title: 'บันทึกแบบร่าง',
      text: 'คุณต้องการบันทึกแบบร่างของการประเมินนี้ใช่หรือไม่',
      showCancelButton: true,
      confirmButtonText: 'ยืนยันการบันทึก',
      cancelButtonText: 'ย้อนกลับ',
      customClass: {
        title: '!swal2-title-mt-20px',
        actions: '!swal2-actions-mt-20px',
        icon: '!swal2-icon-no-border',
        confirmButton: '!swal2-button-bg-green',
        cancelButton: '!swal2-button-bg-gray',
      },
    }).then((result) => {
      if (result.isConfirmed) {
        const body = { ...this.appraisalCompentencyForm.data, apsassessyStatus: '2' }
        this.appraisalService.postCompetency(body).subscribe({
          next: response => {
            if (response.success) {
              Swal.fire({
                title: 'บันทึกสำเร็จ!',
                text: 'การประเมินของคุณถูกบันทึกแล้ว',
                icon: 'success',
                customClass: {
                  confirmButton: '!swal2-button-bg-green',
                }
              });
            } else {
              Swal.fire({
                title: 'ยกเลิก!',
                text: 'การบันทึกถูกยกเลิก',
                icon: 'error',
                customClass: {
                  confirmButton: '!swal2-button-bg-danger',
                }
              });
            }
          }, error: error => {
            Swal.fire({
              title: 'ยกเลิก!',
              text: 'การบันทึกถูกยกเลิก',
              icon: 'error',
              customClass: {
                confirmButton: '!swal2-button-bg-danger',
              }
            });
          }
        })

      } else if (result.dismiss === Swal.DismissReason.cancel) {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        });
      }
    });
  }
  save() {
    Swal.fire({
      iconHtml: `
          <div class="flex items-center justify-center rounded-full !h-80px !w-80px" style="background-color: #E8F8EE;">
              <svg width="39" height="39" viewBox="0 0 39 39" fill="none" xmlns="http://www.w3.org/2000/svg">
                  <rect width="39" height="39" fill="#D2D2D2"/>
                  <g id="Component">
                      <g id="alert cart">
                          <g id="mdi:file-export">
                              <circle cx="22.5" cy="19.5" r="33.5" fill="#E8F8EE"/>
                              <path d="M9.75 3.25C8.88805 3.25 8.0614 3.59241 7.4519 4.2019C6.84241 4.8114 6.5 5.63805 6.5 6.5V32.5C6.5 33.362 6.84241 34.1886 7.4519 34.7981C8.0614 35.4076 8.88805 35.75 9.75 35.75H29.25C30.112 35.75 30.9386 35.4076 31.5481 34.7981C32.1576 34.1886 32.5 33.362 32.5 32.5V13L22.75 3.25M21.125 5.6875L30.0625 14.625H21.125M14.5113 19.8575H26V31.3463L22.555 27.9013L17.9563 32.5L13.3575 27.9013L17.9563 23.3188" 
                              fill="#1DBE5A"/>
                          </g>
                      </g>
                  </g>
              </svg>
          </div>
          `,
      title: 'บันทึกแบบร่าง',
      text: 'คุณต้องการบันทึกแบบร่างของการประเมินนี้ใช่หรือไม่',
      showCancelButton: true,
      confirmButtonText: 'ยืนยันการบันทึก',
      cancelButtonText: 'ย้อนกลับ',
      customClass: {
        title: '!swal2-title-mt-20px',
        actions: '!swal2-actions-mt-20px',
        icon: '!swal2-icon-no-border',
        confirmButton: '!swal2-button-bg-green',
        cancelButton: '!swal2-button-bg-gray',
      },
    }).then((result) => {
      if (result.isConfirmed) {
        const body = { ...this.appraisalCompentencyForm.data, apsassessyStatus: '2' }
        this.appraisalService.postCompetency(body).subscribe({
          next: response => {
            if (response.success) {
              Swal.fire({
                title: 'บันทึกสำเร็จ!',
                text: 'การประเมินของคุณถูกบันทึกแล้ว',
                icon: 'success',
                customClass: {
                  confirmButton: '!swal2-button-bg-green',
                }
              });
            } else {
              Swal.fire({
                title: 'ยกเลิก!',
                text: 'การบันทึกถูกยกเลิก',
                icon: 'error',
                customClass: {
                  confirmButton: '!swal2-button-bg-danger',
                }
              });
            }
          }, error: error => {
            Swal.fire({
              title: 'ยกเลิก!',
              text: 'การบันทึกถูกยกเลิก',
              icon: 'error',
              customClass: {
                confirmButton: '!swal2-button-bg-danger',
              }
            });
          }
        })

      } else if (result.dismiss === Swal.DismissReason.cancel) {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        });
      }
    });
  }

  convertDate(dateInput: string | Date): string {
    const date = typeof dateInput === 'string' ? new Date(dateInput) : dateInput;
    return date.toLocaleDateString('th-TH', { day: 'numeric', month: 'long', year: 'numeric' });
  }

  calnumberCheck() {
    this.appraisalCompentencyForm.data.numberCheck1 = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.filter((y: any) => y.competencyBehavioral.scoreTopicExpectation == 1).length).reduce((acc: any, num: any) => acc + num, 0)
    this.appraisalCompentencyForm.data.numberCheck2 = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.filter((y: any) => y.competencyBehavioral.scoreTopicExpectation == 2).length).reduce((acc: any, num: any) => acc + num, 0)
    this.appraisalCompentencyForm.data.numberCheck3 = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.filter((y: any) => y.competencyBehavioral.scoreTopicExpectation == 3).length).reduce((acc: any, num: any) => acc + num, 0)
    this.appraisalCompentencyForm.data.numberCheck4 = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.filter((y: any) => y.competencyBehavioral.scoreTopicExpectation == 4).length).reduce((acc: any, num: any) => acc + num, 0)
    this.appraisalCompentencyForm.data.numberCheck5 = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.filter((y: any) => y.competencyBehavioral.scoreTopicExpectation == 5).length).reduce((acc: any, num: any) => acc + num, 0)
    this.calWeightTotal()
    this.cdr.detectChanges()

  }
  calWeightScore(numberCheck: number) {
    switch (numberCheck) {
      case (1): {
        return +(this.setting.data.settingScore1) * this.appraisalCompentencyForm.data.numberCheck1
      }
      case (2): {
        return +(this.setting.data.settingScore2) * this.appraisalCompentencyForm.data.numberCheck2
      }
      case (3): {
        return +(this.setting.data.settingScore3) * this.appraisalCompentencyForm.data.numberCheck3
      }
      case (4): {
        return +(this.setting.data.settingScore4) * this.appraisalCompentencyForm.data.numberCheck4
      }
      case (5): {
        return +(this.setting.data.settingScore5) * this.appraisalCompentencyForm.data.numberCheck5
      }
      default: { return }
    }
  }
  calWeightTotal() {
    this.appraisalCompentencyForm.data.weightedTotal = 0
    this.appraisalCompentencyForm.data.weightedTotal = (this.calWeightScore(1) ?? 0) + (this.calWeightScore(2) ?? 0) + (this.calWeightScore(3) ?? 0) + (this.calWeightScore(4) ?? 0) + (this.calWeightScore(5) ?? 0)
    this.cdr.detectChanges()
    this.calAverageScore()
  }
  calAverageScore() {
    const total = this.appraisalCompentencyForm.data.masfromEvaluationAssessment1lList.map((x: any) => x.groupAssessment1.competencyIndicatorsCourses1Mini.behavioralIndicatorsList.length).reduce((acc: any, num: any) => acc + num, 0) * 5
    if (total) {
      this.appraisalCompentencyForm.data.averageScore = 0
      this.appraisalCompentencyForm.data.averageScore = (this.appraisalCompentencyForm.data.weightedTotal / total) * 100
      this.cdr.detectChanges()
    }
  }
  showNumber(text: number | string) {
    const num = Number(text);
    return isNaN(num) ? 0 : +num.toFixed(2);
  }


  calGap(score: number | string) {
    const roundedScore = this.showNumber(score) % 1 <= 0.5 ? Math.floor(this.showNumber(score)) : Math.round(this.showNumber(score));
    if (roundedScore >= 90 && roundedScore <= 100) {
      if (this.appraisalCompentencyForm.data.numberCheck1 || this.appraisalCompentencyForm.data.numberCheck2 || this.appraisalCompentencyForm.data.numberCheck3) {
        return "-1";
      }
      return "+1";
    } else if (roundedScore >= 80 && roundedScore <= 89) {
      if (this.appraisalCompentencyForm.data.numberCheck1 || this.appraisalCompentencyForm.data.numberCheck2 || this.appraisalCompentencyForm.data.numberCheck3) {
        return "-1";
      }
      return "0";
    } else if (roundedScore >= 60 && roundedScore <= 79) {
      return "-1";
    } else if (roundedScore >= 40 && roundedScore <= 59) {
      return "-2";
    } else {
      return "-3";
    }
  }
  calAverage(score: number | string) {
    const roundedScore = this.showNumber(score) % 1 <= 0.5 ? Math.floor(this.showNumber(score)) : Math.round(this.showNumber(score));
    if (roundedScore >= 90 && roundedScore <= 100) {
      return "5";
    } else if (roundedScore >= 80 && roundedScore <= 89) {
      return "4";
    } else if (roundedScore >= 60 && roundedScore <= 79) {
      return "3";
    } else if (roundedScore >= 40 && roundedScore <= 59) {
      return "2";
    } else {
      return "1";
    }
  }
}